"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FunctionLikeSchema = void 0;
function _chalk() {
  const data = _interopRequireDefault(require("chalk"));
  _chalk = function () {
    return data;
  };
  return data;
}
function _schemaNode() {
  const data = require("../schema-node");
  _schemaNode = function () {
    return data;
  };
  return data;
}
function _parameter() {
  const data = require("./parameter");
  _parameter = function () {
    return data;
  };
  return data;
}
function _docs() {
  const data = require("./docs");
  _docs = function () {
    return data;
  };
  return data;
}
function _tag() {
  const data = require("./docs/tag");
  _tag = function () {
    return data;
  };
  return data;
}
function _schemaRegistry() {
  const data = require("../schema-registry");
  _schemaRegistry = function () {
    return data;
  };
  return data;
}
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/**
 * function-like can be a function, method, arrow-function, variable-function, etc.
 */
class FunctionLikeSchema extends _schemaNode().SchemaNode {
  constructor(location, name, params, returnType, signature, modifiers = [], doc, typeParams, decorators) {
    super();
    this.location = location;
    this.name = name;
    this.modifiers = modifiers;
    this.typeParams = typeParams;
    this.decorators = decorators;
    this.params = params;
    this.returnType = returnType;
    this.doc = doc;
    this.signature = signature || FunctionLikeSchema.createSignature(this.name, this.params, this.returnType);
  }
  getNodes() {
    return [...this.params, this.returnType, ...(this.decorators || [])];
  }
  toString(options) {
    const bold = options?.color ? _chalk().default.bold : text => text;
    const paramsStr = this.params.map(param => param.toString()).join(', ');
    const typeParamsStr = this.typeParams ? `<${this.typeParams.join(', ')}>` : '';
    const decoratorsStr = this.decorators?.map(decorator => decorator.toString(options)).join('\n');
    return `${this.decorators ? `${decoratorsStr}\n` : ''}${this.modifiersToString()}${typeParamsStr}${bold(this.name)}(${paramsStr}): ${this.returnType.toString(options)}`;
  }
  toFullSignature(options) {
    let result = '';
    if (options?.showDocs && this.doc) {
      result += `${this.doc.toFullSignature()}\n`;
    }
    const decoratorsStr = this.decorators?.map(decorator => decorator.toString()).join('\n');
    if (decoratorsStr) {
      result += `${decoratorsStr}\n`;
    }
    const modifiersStr = this.modifiersToString();
    const typeParamsStr = this.typeParams ? `<${this.typeParams.join(', ')}>` : '';
    const paramsStr = this.params.map(param => {
      let paramStr = '';
      if (param.isSpread) {
        paramStr += '...';
      }
      paramStr += param.name;
      if (param.isOptional) {
        paramStr += '?';
      }
      paramStr += `: ${param.type.toString()}`;
      if (param.defaultValue !== undefined) {
        paramStr += ` = ${param.defaultValue}`;
      }
      return paramStr;
    }).join(', ');
    result += `${modifiersStr}${this.name}${typeParamsStr}(${paramsStr}): ${this.returnType.toString()}`;
    return result;
  }
  isDeprecated() {
    return Boolean(this.doc?.hasTag(_tag().TagName.deprecated));
  }
  isPrivate() {
    return Boolean(this.modifiers.find(m => m === 'private') || this.doc?.hasTag(_tag().TagName.private));
  }
  generateSignature() {
    return FunctionLikeSchema.createSignature(this.name, this.params, this.returnType, this.decorators);
  }
  static createSignature(name, params, returnType, decorators) {
    const paramsStr = params.map(param => {
      let type = param.type.toString();
      if (param.isSpread) type = `...${type}`;
      return `${param.name}${param.isOptional ? '?' : ''}: ${type}`;
    }).join(', ');
    const decoratorsStr = decorators?.map(decorator => decorator.toString()).join('\n');
    return `${decorators ? `${decoratorsStr}\n` : ''}${name}(${paramsStr}): ${returnType.toString()}`;
  }
  toObject() {
    return {
      ...super.toObject(),
      name: this.name,
      params: this.params.map(param => param.toObject()),
      returnType: this.returnType.toObject(),
      signature: this.signature,
      modifiers: this.modifiers,
      doc: this.doc?.toObject(),
      typeParams: this.typeParams,
      decorators: this.decorators?.map(decorator => decorator.toObject())
    };
  }
  static fromObject(obj) {
    return new FunctionLikeSchema(obj.location, obj.name, obj.params.map(param => _parameter().ParameterSchema.fromObject(param)), _schemaRegistry().SchemaRegistry.fromObject(obj.returnType), obj.signature, obj.modifiers, obj.doc ? _docs().DocSchema.fromObject(obj.doc) : undefined, obj.typeParams, obj.decorators?.map(decorator => _schemaRegistry().SchemaRegistry.fromObject(decorator)));
  }
  modifiersToString() {
    const modifiersToPrint = this.modifiers.filter(modifier => modifier !== 'export');
    return modifiersToPrint.length ? `${modifiersToPrint.join(' ')} ` : '';
  }
}
exports.FunctionLikeSchema = FunctionLikeSchema;

//# sourceMappingURL=function-like.js.map