"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TypeRefSchema = void 0;
function _component() {
  const data = require("@teambit/component");
  _component = function () {
    return data;
  };
  return data;
}
function _chalk() {
  const data = _interopRequireDefault(require("chalk"));
  _chalk = function () {
    return data;
  };
  return data;
}
function _schemaNode() {
  const data = require("../schema-node");
  _schemaNode = function () {
    return data;
  };
  return data;
}
function _schemaRegistry() {
  const data = require("../schema-registry");
  _schemaRegistry = function () {
    return data;
  };
  return data;
}
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/**
 * can be one of the following:
 * 1. a reference to another "export" in the same component
 * 2  a reference to another declaration in the same file (internal)
 * 3. a reference to another component.
 * 4. a reference to a package.
 */
class TypeRefSchema extends _schemaNode().SchemaNode {
  /**
   *  optional type arguments, e.g. type Foo = Bar<X, Y>. The X and Y are type arguments.
   */

  constructor(location,
  /**
   * name of the reference to type.
   */
  name,
  /**
   * target component id. existing if the type is defined in another component.
   */
  componentId,
  /**
   * target package name. existing if type is defined in different package.
   */
  packageName,
  /**
   * if the reference is not exported from the component, it can be internal to the file.
   */
  internalFilePath) {
    super();
    this.location = location;
    this.name = name;
    this.packageName = packageName;
    this.internalFilePath = internalFilePath;
    this.componentId = componentId;
  }
  getNodes() {
    return this.typeArgs || [];
  }
  withTypeArgs(typeArgs) {
    this.typeArgs = typeArgs;
    return this;
  }
  toString(options) {
    const name = this.nameToString(options);
    if (!this.typeArgs) {
      return name;
    }
    const args = this.typeArgs.map(arg => arg.toString(options)).join(', ');
    return `${name}<${args}>`;
  }
  nameToString(options) {
    const dim = options?.color ? _chalk().default.dim : str => str;
    if (this.componentId) {
      const compStr = dim(`(component: ${this.componentId.toStringWithoutVersion()})`);
      return `${compStr} ${this.name}`;
    }
    if (this.packageName) {
      const pkgStr = dim(`(package: ${this.packageName})`);
      return `${pkgStr} ${this.name}`;
    }
    return this.name;
  }
  toFullSignature(options) {
    let signature = this.name;
    if (this.typeArgs && this.typeArgs.length > 0) {
      const argsSignatures = this.typeArgs.map(arg => arg.toFullSignature(options)).join(', ');
      signature += `<${argsSignatures}>`;
    }
    if (options?.showDocs && this.doc) {
      const docString = this.doc.toFullSignature();
      signature = `${docString}\n${signature}`;
    }
    return signature;
  }

  /**
   * whether this type was already exported in this component
   */
  isFromThisComponent() {
    return !this.componentId && !this.packageName;
  }

  /**
   * whether this type is internal to the file.
   */
  isInternalReference() {
    return !!this.internalFilePath && this.isFromThisComponent();
  }

  /**
   * whether this type is exported from this component.
   */
  isExported() {
    return this.isFromThisComponent() && !this.isInternalReference();
  }
  toObject() {
    return {
      ...super.toObject(),
      name: this.name,
      componentId: this.componentId ? this.componentId.toObject() : undefined,
      packageName: this.packageName,
      internalFilePath: this.internalFilePath,
      typeArgs: this.typeArgs?.map(type => type.toObject())
    };
  }
  static fromObject(obj) {
    const location = obj.location;
    const name = obj.name;
    let componentId;
    try {
      componentId = obj.componentId ? _component().ComponentID.fromObject(obj.componentId) : undefined;
    } catch {
      componentId = undefined;
    }
    const packageName = obj.packageName;
    const internalFilePath = obj.internalFilePath;
    const typeArgs = obj.typeArgs?.map(type => _schemaRegistry().SchemaRegistry.fromObject(type));
    return new TypeRefSchema(location, name, componentId, packageName, internalFilePath).withTypeArgs(typeArgs);
  }
}
exports.TypeRefSchema = TypeRefSchema;

//# sourceMappingURL=type-ref.js.map